@@ -1,4 +1,11 @@
 from .helpers import from_client
 from .statebuilder import StateBuilder, ChainedStateBuilder
 from .layers import SegmentationLayerConfig, AnnotationLayerConfig, ImageLayerConfig
-from .mappers import SelectionMapper, PointMapper, LineMapper, SphereMapper, BoundingBoxMapper
+from .mappers import (
+    SelectionMapper,
+    PointMapper,
+    LineMapper,
+    SphereMapper,
+    BoundingBoxMapper,
+    SplitPointMapper,
+)
