@@ -2,6 +2,26 @@ import numpy as np
 from nglui import annotation
 import pandas as pd
 from collections.abc import Collection
+from itertools import chain
+
+
+def _multipoint_transform(row, pt_columns, squeeze_cols):
+    """Reshape dataframe to accomodate multiple points in a single row"""
+    pts = {pcol: np.atleast_2d(row[pcol]) for pcol in pt_columns}
+    n_pts = pts[pt_columns[0]].shape[0]
+    rows = [{} for _ in range(n_pts)]
+    for col in row.index:
+        if col in pt_columns:
+            if col in squeeze_cols:
+                for r, v in zip(rows, pts[col].squeeze().tolist()):
+                    r[col] = v
+            else:
+                for r, v in zip(rows, pts[col].tolist()):
+                    r[col] = v
+        else:
+            for r in rows:
+                r[col] = row[col]
+    return rows
 
 
 class SelectionMapper(object):
@@ -88,6 +108,7 @@ class AnnotationMapperBase(object):
         set_position,
         gather_linked_segmentations,
         share_linked_segmentations,
+        multipoint,
     ):
 
         self._config = dict(
@@ -101,6 +122,7 @@ class AnnotationMapperBase(object):
             set_position=set_position,
             gather_linked_segmentations=gather_linked_segmentations,
             share_linked_segmentations=share_linked_segmentations,
+            multipoint=multipoint,
         )
         self._tag_map = None
 
@@ -140,6 +162,22 @@ class AnnotationMapperBase(object):
     def set_position(self):
         return self._config.get("set_position", False)
 
+    @property
+    def multipoint(self):
+        return self._config.get("multipoint", False)
+
+    def multipoint_reshape(self, data, pt_columns, squeeze_cols=[]):
+        if data is None or len(data) == 0:
+            return data
+        else:
+            rows = data.apply(
+                lambda x: _multipoint_transform(
+                    x, pt_columns=pt_columns, squeeze_cols=squeeze_cols
+                ),
+                axis=1,
+            ).tolist()
+            return pd.DataFrame.from_records([r for r in chain.from_iterable(rows)])
+
     @property
     def tag_map(self):
         if self._tag_map is None:
@@ -222,7 +260,7 @@ class AnnotationMapperBase(object):
 
     def _get_position(self, data):
         if len(data) > 0 and self.set_position is True:
-            return list(data[self.data_columns[0]].iloc[0])
+            return list(np.atleast_2d(data[self.data_columns[0]].iloc[0])[0])
         else:
             return None
 
@@ -250,6 +288,8 @@ class PointMapper(AnnotationMapperBase):
     set_position : bool, optional
         If set to True, moves the position to center on the first point in the
         data.
+    multipoint: bool, optional
+        If True, permits multiple points in a given row, sharing data in other columns. Default is False.
     """
 
     def __init__(
@@ -262,6 +302,7 @@ class PointMapper(AnnotationMapperBase):
         gather_linked_segmentations=True,
         share_linked_segmentations=False,
         set_position=False,
+        multipoint=False,
     ):
         super(PointMapper, self).__init__(
             type="point",
@@ -273,6 +314,7 @@ class PointMapper(AnnotationMapperBase):
             gather_linked_segmentations=gather_linked_segmentations,
             share_linked_segmentations=share_linked_segmentations,
             set_position=set_position,
+            multipoint=multipoint,
         )
 
     def _default_array_data_columns(self):
@@ -281,6 +323,10 @@ class PointMapper(AnnotationMapperBase):
     def _render_data(self, data):
         if self.array_data:
             data = pd.DataFrame(data={"pt": np.array(data).tolist()})
+
+        if self.multipoint:
+            data = self.multipoint_reshape(data, pt_columns=self.data_columns)
+
         col = self.data_columns[0]
         relinds = ~pd.isnull(data[col])
 
@@ -325,6 +371,9 @@ class LineMapper(AnnotationMapperBase):
         Rows with the same non-NaN value will be collected into a grouped annotation.
     set_position : bool, optional
         If set to True, moves the position to center on the first point in the data (using point_column_a).
+    multipoint: bool, optional
+        If True, permits multiple points in a given row, sharing data in other columns.
+        Each point row must have the same number of points. Default is False.
     """
 
     def __init__(
@@ -338,6 +387,7 @@ class LineMapper(AnnotationMapperBase):
         gather_linked_segmentations=True,
         share_linked_segmentations=False,
         set_position=False,
+        multipoint=False,
     ):
         super(LineMapper, self).__init__(
             type="line",
@@ -349,6 +399,7 @@ class LineMapper(AnnotationMapperBase):
             gather_linked_segmentations=gather_linked_segmentations,
             share_linked_segmentations=share_linked_segmentations,
             set_position=set_position,
+            multipoint=multipoint,
         )
 
     def _default_array_data_columns(self):
@@ -359,8 +410,11 @@ class LineMapper(AnnotationMapperBase):
             data = pd.DataFrame(
                 data={"pt_a": data[0].tolist(), "pt_b": data[1].tolist()}
             )
-        colA, colB = self.data_columns
 
+        if self.multipoint:
+            data = self.multipoint_reshape(data, pt_columns=self.data_columns)
+
+        colA, colB = self.data_columns
         relinds = np.logical_and(~pd.isnull(data[colA]), ~pd.isnull(data[colB]))
 
         ptAs = np.vstack(data[colA][relinds])
@@ -402,6 +456,9 @@ class SphereMapper(AnnotationMapperBase):
         Rows with the same non-NaN value will be collected into a grouped annotation.
     set_position : bool, optional
         If set to True, moves the position to center on the first point in the data.
+    multipoint: bool, optional
+        If True, permits multiple points in a given row, sharing data in other columns.
+        Each point row must have the same number of points. Default is False.
     """
 
     def __init__(
@@ -416,6 +473,7 @@ class SphereMapper(AnnotationMapperBase):
         share_linked_segmentations=False,
         z_multiplier=0.1,
         set_position=False,
+        multipoint=False,
     ):
         super(SphereMapper, self).__init__(
             type="sphere",
@@ -427,6 +485,7 @@ class SphereMapper(AnnotationMapperBase):
             gather_linked_segmentations=gather_linked_segmentations,
             share_linked_segmentations=share_linked_segmentations,
             set_position=set_position,
+            multipoint=multipoint,
         )
         self._z_multiplier = z_multiplier
 
@@ -436,7 +495,13 @@ class SphereMapper(AnnotationMapperBase):
     def _render_data(self, data):
         if self.array_data:
             data = pd.DataFrame(data={"ctr_pt": data[0].tolist(), "rad": data[1]})
+
         col_ctr, col_rad = self.data_columns
+        if self.multipoint:
+            data = self.multipoint_reshape(
+                data, pt_columns=self.data_columns, squeeze_cols=[col_rad]
+            )
+
         relinds = np.logical_and(~pd.isnull(data[col_ctr]), ~pd.isnull(data[col_rad]))
 
         pts = np.vstack(data[col_ctr][relinds])
@@ -475,6 +540,7 @@ class BoundingBoxMapper(AnnotationMapperBase):
         gather_linked_segmentations=True,
         share_linked_segmentations=False,
         set_position=False,
+        multipoint=False,
     ):
         super(BoundingBoxMapper, self).__init__(
             type="axis_aligned_bounding_box",
@@ -486,6 +552,7 @@ class BoundingBoxMapper(AnnotationMapperBase):
             gather_linked_segmentations=gather_linked_segmentations,
             share_linked_segmentations=share_linked_segmentations,
             set_position=set_position,
+            multipoint=multipoint,
         )
 
     def _default_array_data_columns(self):
@@ -496,8 +563,11 @@ class BoundingBoxMapper(AnnotationMapperBase):
             data = pd.DataFrame(
                 data={"pt_a": data[0].tolist(), "pt_b": data[1].tolist()}
             )
-        colA, colB = self.data_columns
 
+        if self.multipoint:
+            data = self.multipoint_reshape(data, pt_columns=self.data_columns)
+
+        colA, colB = self.data_columns
         relinds = np.logical_and(~pd.isnull(data[colA]), ~pd.isnull(data[colB]))
 
         ptAs = np.vstack(data[colA][relinds])
